﻿using W803Demo.Common;
using W803Demo.Data;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using System.Net.Http;
using System.Xml.Linq;
using System.Threading.Tasks;
using System.Threading;

// The Item Detail Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234232

namespace W803Demo
{
    /// <summary>
    /// A page that displays details for a single item within a group.
    /// </summary>
    public sealed partial class ItemDetailPage : Page
    {
        private NavigationHelper navigationHelper;
        private ObservableDictionary defaultViewModel = new ObservableDictionary();

        /// <summary>
        /// NavigationHelper is used on each page to aid in navigation and 
        /// process lifetime management
        /// </summary>
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        /// <summary>
        /// This can be changed to a strongly typed view model.
        /// </summary>
        public ObservableDictionary DefaultViewModel
        {
            get { return this.defaultViewModel; }
        }

        private CancellationTokenSource cancellationTokenSource;

        public ItemDetailPage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += navigationHelper_LoadState;
            this.navigationHelper.SaveState += navigationHelper_SaveState;
        }

        private void navigationHelper_SaveState(object sender, SaveStateEventArgs e)
        {
            this.cancellationTokenSource.Cancel();
        }

        private async void navigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
            // TODO: Create an appropriate data model for your problem domain to replace the sample data
            var item = await SampleDataSource.GetItemAsync((String)e.NavigationParameter);
            this.DefaultViewModel["Item"] = item;

            this.cancellationTokenSource = new CancellationTokenSource();

            //LazyLoadData();
            //HeavyLoadData();
            //LoadData2();
            try
            {
                LoadData3(this.cancellationTokenSource.Token);
            }
            catch (OperationCanceledException)
            {
                // Posso ignorare
            }
        }

        private void LazyLoadData()
        {
            this.grid.ItemsSource = new LazyCollection<SampleDataItem>(LazyLoad);
        }

        private async Task<IEnumerable<SampleDataItem>> LazyLoad(int start, int count)
        {
            await Task.Delay(1000);

            return Enumerable.Range(start, count)
               .Select(s => GetDummyItem(s))
               .ToArray();
        }

        private void HeavyLoadData()
        {
            var source = Enumerable.Range(0, 1000000)
                .Select(s => GetDummyItem(s))
                .ToArray();

            this.grid.ItemsSource = source;
        }

        private async void LoadData1()
        {
            HttpClient client = new HttpClient();
            using (var stream = await client.GetStreamAsync("http://www.communitydays.it/rss/").ConfigureAwait(false))
            {
                var doc = XDocument.Load(stream);
                // ecc.
                var source = new [] {"item1", "item2"};

                this.grid.ItemsSource = source;
            }
        }

        private async void LoadData2()
        {
            // No ConfigureAwait
            var source = await LoadData2FromHttp();

            this.grid.ItemsSource = source;
        }

        private async void LoadData3(CancellationToken token)
        {
            // No ConfigureAwait
            var source = await LoadData3FromHttp(token);

            this.grid.ItemsSource = source;
        }

        private static async Task<object> LoadData2FromHttp()
        {
            HttpClient client = new HttpClient();
            using (var stream = await client.GetStreamAsync("http://www.communitydays.it/rss/").ConfigureAwait(false))
            {
                var doc = XDocument.Load(stream);
                // ecc.
                // IMPORTANTE: materializzare
                var source = Enumerable.Range(1, 100).Select(GetDummyItem).ToArray();

                return source;
            }
        }

        private static async Task<object> LoadData3FromHttp(CancellationToken token)
        {
            HttpClient client = new HttpClient();
            using (var response = await client.GetAsync("http://www.communitydays.it/rss/", token).ConfigureAwait(false))
            {
                using (var stream = await response.Content.ReadAsStreamAsync())
                {
                    var doc = XDocument.Load(stream);
                    // ecc.
                    // IMPORTANTE: materializzare
                    var source = Enumerable.Range(1, 100).Select(GetDummyItem).ToArray();

                    await Task.Delay(2000, token);

                    // TODO: monitorare
                    // token.IsCancellationRequested
                    token.ThrowIfCancellationRequested();

                    return source;
                }
            }
        }

        private static SampleDataItem GetDummyItem(int s)
        {
            return new SampleDataItem(s.ToString(), "Title " + s.ToString(), "", "http://lorempixel.com/100/100/technics/?" + s, "", "");
        }

        #region NavigationHelper registration

        /// The methods provided in this section are simply used to allow
        /// NavigationHelper to respond to the page's navigation methods.
        /// 
        /// Page specific logic should be placed in event handlers for the  
        /// <see cref="GridCS.Common.NavigationHelper.LoadState"/>
        /// and <see cref="GridCS.Common.NavigationHelper.SaveState"/>.
        /// The navigation parameter is available in the LoadState method 
        /// in addition to page state preserved during an earlier session.


        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedTo(e);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedFrom(e);
        }

        #endregion
    }
}