﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml.Data;

namespace W803Demo
{
    public class LazyCollection<T> : ObservableCollection<T>, ISupportIncrementalLoading
    {

        private Func<int, int, Task<IEnumerable<T>>> fetch;

        public LazyCollection(Func<int, int, Task<IEnumerable<T>>> fetch)
        {
            this.fetch = fetch;
        }

        bool ISupportIncrementalLoading.HasMoreItems
        {
            get { return true; }
        }

        Windows.Foundation.IAsyncOperation<LoadMoreItemsResult> ISupportIncrementalLoading.LoadMoreItemsAsync(uint count)
        {
            return LoadTask(count).AsAsyncOperation();
        }

        private async Task<LoadMoreItemsResult> LoadTask(uint count)
        {
            Debug.WriteLine("Load " + count);

            var r = await fetch(this.Count, (int)count);
            foreach (var item in r)
            {
                this.Add(item);
            }

            return new LoadMoreItemsResult
            {
                Count = (uint)r.Count(),
            };
        }
    }
}
